<?php

/**
 * TM::Gallery
 *
 * Uses ImageMagick's convert to create a gallery (including downsized images,
 * thumbnails and HTML source) from a directory of JPG files.
 */

/*
 * HISTORY:
 * 2003-08-28
 * - Source for thumbnail is the already scaled image, not the original.
 * - Create files only if they not exist.
 * - Comments added.
 * KNOWN BUGS/TODO:
 * - 99x99> does not work (never upscale images!).
 */

$directory = "C:/Directory with the photos to be scaled and added/";
$size = 720;
$thumbsSize = 180;

// Create directory if still missing.
if (! file_exists(".thumbs")) mkdir(".thumbs", 0701);

echo "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">
<html lang=\"de\">
 <head>
  <meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">
  <title>TM::Gallery</title>
  <meta name=\"author\" content=\"Thiemo Mttig\">
  <meta name=\"robots\" content=\"noindex,nofollow\">
 </head>
 <body>

  <h1>TM::Gallery</h1>

  <p>
";

// Run forever, if needed (convert takes time).
set_time_limit(0);

$filenames = array();
$dp = opendir($directory);
while ($filename = readdir($dp))
{
    // Skip ".", ".." and all non-JPG files.
    if (! preg_match('/\.jpe?g$/i', $filename)) continue;

    $filenames[] = $filename;
}
closedir($dp);
sort($filenames);

$fp = fopen("fotos.csv", "w");
fwrite($fp, "id,dateiname,zeit,kamera,bytes,breite,hoehe,vorschaubreite,vorschauhoehe\n");

foreach ($filenames as $filename)
{
    $source = $directory . $filename;
    $target = $filename;
    $target = preg_replace('/\s+/', '_', $target);

    // "Original".jpg becomes "640".jpg if still missing.
    if (! file_exists($target))
    {
        $cmd = "C:\\Programme\\ImageMagick-5.4.9-Q16\\convert";
        $cmd .= " -size " . $size . "x" . $size . " -resize " . $size . "x" . $size;
        $cmd .= " +profile \"*\" -quality 80 -interlace plane";
        $cmd .= " \"" . $source . "\" " . $target;
        exec($cmd);
    }
    // "640".jpg becomes .thumbs/"80".jpg if still missing (original not used!).
    if (! file_exists(".thumbs\\" . $target))
    {
        $cmd = "C:\\Programme\\ImageMagick-5.4.9-Q16\\convert";
        $cmd .= " -size " . $thumbsSize . "x" . $thumbsSize . " -resize " . $thumbsSize . "x" . $thumbsSize;
        $cmd .= " +profile \"*\" -quality 50";
        $cmd .= " \"" . $target . "\" .thumbs\\" . $target;
        exec($cmd);
    }

    $time = substr($filename, 0, 19);
    $time = substr_replace($time, ':', 13, 1);
    $time = substr_replace($time, ':', 16, 1);
    $timestamp = strtotime($time);

    $exif = read_exif_data($directory . $filename);
    $camera = trim($exif['CameraModel']);
    $camera = str_replace('FinePix', 'FUJI FinePix', $camera);
    $camera = str_replace(' DIGITAL', '', $camera);

    $id = substr($target, -12, 8);

    // Alt.
    $alt = date('j.n. G:i', $timestamp);
    $title = date('j.n., G:i', $timestamp);
    $title .= ' Uhr [' . $camera . ']';
    // Title becomes "Description (99k)" for example.
    $title .= " (" . round(filesize($target) / 1024) . "k)";

    list($breite, $hoehe) = getimagesize($target);
    list($vorschaubreite, $vorschauhoehe, , $attributes) =
        getimagesize('.thumbs/' . $target);

    // View starts here.
    $html = '<a href="' . $target . '">';
    $html .= '<img src=".thumbs/' . $target . '"';
    if ($alt) $html .= ' alt="' . $alt. '"';
    if ($title) $html .= ' title="' . $title. '"';
    $html .= ' ' . $attributes;
    $html .= '';
    $html .= "></a>\n";
    echo $html;

    // Force output (some kind of progress bar).
    flush();

    fwrite($fp, "$id,$target," . date("Y-m-d H:i:s", $timestamp) .
        ",$camera," . filesize($target) . ",$breite,$hoehe,$vorschaubreite,$vorschauhoehe\n");
}

echo "</p>\n";

?>